﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Ant_Colony_Optimalization_Parallel.Graph
{
    public class CNode : IEquatable<CNode>
    {
        public const double ALPHA = 3;
        public const double BETA = 2;

        public int id { get; set; }
        public CEdge[] EdgeArray { get; set; }
        public CNode[] NeighborNodeIdArray { get; set; }
        public double[] ProbabilityArray { get; set; }
        public double[] CumulProbabilityArray { get; set; }
        public CPoint Point { get; set; }

        public int edgeCount { get; set; }

        public CNode(int id, CPoint point, int edgeCount)
        {
            this.id = id;
            this.Point = point;
            this.EdgeArray = new CEdge[edgeCount];
            this.NeighborNodeIdArray = new CNode[edgeCount];

            this.edgeCount = 0;
        }

        public void AddEdge(CEdge edge)
        {
            this.EdgeArray[this.edgeCount] = edge;
            this.edgeCount++;
           
        }

        public void RecalculateNeighbor()
        {
            this.NeighborNodeIdArray = new CNode[this.EdgeArray.Length];
            for (int i = 0; i < edgeCount; i++)
            {
                this.NeighborNodeIdArray[i] = this.EdgeArray[i].GetNeighbor(id);
            }
        }

        public void RecalculateProbabilityArray()
        {
            this.ProbabilityArray = new double[this.EdgeArray.Count()];
            this.CumulProbabilityArray = new double[this.EdgeArray.Count()];

            for (int i = 0; i < this.EdgeArray.Count(); i++)
            {
                CEdge edge = this.EdgeArray[i];
                this.ProbabilityArray[i] = this.CalculateTauete(edge.Pheromone, edge.Distance);
            }
        }

        public void RecalculateCumulProbabilityArray()
        {
            double arraySum = this.ProbabilityArray.Sum();


            this.CumulProbabilityArray[0] = this.ProbabilityArray[0] / arraySum;
            for (int i = 1; i < this.ProbabilityArray.Count(); i++)
            {
                double i1 = this.CumulProbabilityArray[i - 1];
                double i2 = (this.ProbabilityArray[i] / arraySum);
                this.CumulProbabilityArray[i] = i1 + i2;
            }
        }

        public double CalculateTauete(double pheromone, double distance)
        {
            double tauete = System.Math.Pow(pheromone, ALPHA) * System.Math.Pow((1 / distance), BETA);
            return tauete;
        }

        public string ToString()
        {
            return this.id.ToString();
        }

        public bool Equals(CNode other)
        {
            return this.Point.Equals(other.Point);
        }

        public void SetEdgeCount(int count)
        {
            this.EdgeArray = new CEdge[count];
        }

        public void RemoveNullNodes()
        {
            this.EdgeArray = this.EdgeArray.Where(x => x != null).ToArray();
        }

        public void RemoveEdge(CEdge edge)
        {
            for (int i = 0; i < this.EdgeArray.Count(); i++)
            {
                if (this.EdgeArray[i] != null)
                {
                    if (this.EdgeArray[i].Equals(edge))
                    {
                        this.EdgeArray[i] = null;
                    }
                }
            }
        }

        public string ToStringExport()
        {
            return "N " + this.Point.X + " " + this.Point.Y;
        }

        public string ToStringExportResult()
        {
            return "NR " + this.Point.X + " " + this.Point.Y;
        }


    }
}
