﻿using Ant_Colony_Optimalization_Parallel.Ant;
using Ant_Colony_Optimalization_Parallel.Graph;
using Ant_Colony_Optimalization_Parallel.Other;
using System;
using System.Collections.Generic;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;

namespace Ant_Colony_Optimalization_Parallel
{
    class Program
    {

        static void Main(string[] args)
        {
            LinkedList<string> resultList = new LinkedList<string>();

            for (int w = 0; w < 1; w++)
            {
                //načtení nastavení do proměnných
                int iterationCount = Properties.Settings.Default.IterationCount;
                int noChangeFor = Properties.Settings.Default.StopIfNoChange;
                string inputFile = Properties.Settings.Default.InputFile;
                string outputFile = Properties.Settings.Default.OutputFile;
                int bestAntCount = Properties.Settings.Default.BestAntsCount;
                double alpha = Properties.Settings.Default.Alfa;
                double beta = Properties.Settings.Default.Beta;
                double pheromoneMin = Properties.Settings.Default.PheromoneMin;
                double pheromoneMax = Properties.Settings.Default.PheromoneMax;
                double pheromoneDecrease = Properties.Settings.Default.PheromoneDecrease;
                double pheromoneIncrease = Properties.Settings.Default.PheromoneIncrase;
                int threadCount = Properties.Settings.Default.MaxThreadCount;

                CGraphObjectHolder objectHolder = new CGraphObjectHolder();
                DateTime startTime = DateTime.Now;

                //načtení vstupních dat ze souboru
                string[] lines = System.IO.File.ReadAllLines(inputFile);
                List<CNode> nodeList = new List<CNode>();
                int counter = 0;
                foreach (string line in lines)
                {
                    if (line.StartsWith("N "))
                    {
                        string[] pointArray = line.Split(' ');
                        CPoint point = new CPoint(Convert.ToInt32(pointArray[1]), Convert.ToInt32(pointArray[2]));
                        nodeList.Add(new CNode(counter, point, 0));
                        counter++;
                    }
                }
                objectHolder.NormalNodes = nodeList.ToArray();

                //nastavení velikosti pole hran pro uzly
                foreach (CNode node in objectHolder.NormalNodes)
                {
                    node.SetEdgeCount(objectHolder.NormalNodes.Length - 1);
                }

                //vytvoření hran
                int counter2 = 0;
                int count = (objectHolder.NormalNodes.Length * (objectHolder.NormalNodes.Length - 1)) / 2;
                CEdge[] edgeArray = new CEdge[count];
                for (int i = 0; i < objectHolder.NormalNodes.Length; i++)
                {
                    for (int j = i + 1; j < objectHolder.NormalNodes.Length; j++)
                    {
                        edgeArray[counter2] = new CEdge(counter2, objectHolder.NormalNodes[i], objectHolder.NormalNodes[j]);
                        counter2++;
                    }
                }
                objectHolder.AllEdges = edgeArray;

                //každému uzlu se zjisti soused na hraně
                foreach (CNode node in objectHolder.NormalNodes)
                {
                    node.RecalculateNeighbor();
                }

                Console.WriteLine("Node count : " + objectHolder.NormalNodes.Count());
                Console.WriteLine("edge count : " + objectHolder.AllEdges.Count());

                //výchozí hodnoty výpočtu
                double best = double.MaxValue;
                int noChange = 0;
                CAntCompleteGraph bestAnt = null;

                //vytvoření mravenců
                CAntCompleteGraph[] antArray = new CAntCompleteGraph[objectHolder.NormalNodes.Length];
                for (int j = 0; j < objectHolder.NormalNodes.Length; j++)
                {
                    CAntCompleteGraph acg = new CAntCompleteGraph(objectHolder.NormalNodes[j], objectHolder.NormalNodes.Length, alpha, beta, pheromoneIncrease);
                    antArray[j] = acg;
                }

                //nastevení počtu vláken
                ParallelOptions opt = new ParallelOptions();
                opt.MaxDegreeOfParallelism = threadCount;

                //výpočet
                for (int t = 0; t < iterationCount; t++)
                {
                    //reset mravenců
                    Parallel.ForEach(antArray, opt, ant =>
                    {
                        ant.Reset();
                    });

                    //hledání cesty mravenci
                    Parallel.ForEach(antArray, opt, ant =>
                    {
                        while (ant.MoveToNextNode())
                        {

                        }
                    });

                    //seřazení dle dosažené cesty
                    Array.Sort(antArray);


                    //nejlepší zvyšují feromon
                    for (int k = 0; k < bestAntCount; k++)
                    {
                        //pokud nejsou horší než
                        //if (antArray[k].Distance < (best * 1.03))
                        {
                            antArray[k].ActualizePheromone();
                            new CDecreaseMultiply(pheromoneDecrease, pheromoneMin).DecreasePheromone(objectHolder.AllEdges);
                        }
                    }


                    //pokud je nejlepší tak se uloží
                    if (antArray[0].Distance < best)
                    {
                        best = antArray[0].Distance;
                        bestAnt = antArray[0];
                        noChange = 0;
                    }
                    else
                    {
                        //zvýší se počet bez změny
                        noChange++;
                    }

                    Console.WriteLine(t + " ;    " + best + " ;  " + (DateTime.Now - startTime).ToString());
                    resultList.AddLast(best.ToString());
                    //pokud se dlouho nezmění nejlepší cesta ukončí se výpočet
                    if (noChange == noChangeFor)
                    {
                        t = int.MaxValue - 1;
                    }

                }

                Console.WriteLine("final ;    " + best + " ;  " + (DateTime.Now - startTime).ToString());
                //resultList.AddLast("final ;    " + best + " ;  " + (DateTime.Now - startTime).ToString());
                //zápis výsledků
                using (System.IO.StreamWriter file = new System.IO.StreamWriter(outputFile))
                {
                    foreach (CNode node in bestAnt.VisitedNodes)
                    {
                        file.WriteLine(node.ToStringExportResult());
                    }
                    //Console.WriteLine("OK");
                }

            }
            Console.ReadLine();
        }
    }
}
