﻿using Ant_Colony_Optimalization.Graph;
using Ant_Colony_Optimalization.Graphic.Object;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Ant_Colony_Optimalization.Generator
{
    public class CEdgeGenerator
    {
        public CEdgeGenerator()
        {
        }

        public LinkedList<CEdge> CalculateEdges(CNode[,] nodeArray)
        {
            double shorter = this.CalculateDistance(nodeArray[0, 0], nodeArray[1, 1]);
            shorter = shorter + shorter / 2;

            foreach (CNode node in nodeArray)
            {
                node.SetEdgeCount(8);
            }

            int counter = 0;

            int xMax = nodeArray.GetLength(0);
            int yMax = nodeArray.GetLength(1);

            LinkedList<CEdge> edgeList = new LinkedList<CEdge>();
            for (int i = 0; i < xMax; i++)
            {
                for (int j = 0; j < yMax; j++)
                {
                    CNode nodeFrom = nodeArray[i, j];

                    CEdge edge;

                    if (i + 1 < xMax)
                    {
                        edge = new CEdge(counter, nodeArray[i, j], nodeArray[i + 1, j]);
                        edgeList.AddLast(edge);
                        counter++;
                    }
                    if (j + 1 < yMax)
                    {
                        edge = new CEdge(counter, nodeArray[i, j], nodeArray[i, j + 1]);
                        edgeList.AddLast(edge);
                        counter++;
                    }
                    if (i + 1 < xMax && j + 1 < yMax)
                    {
                        edge = new CEdge(counter, nodeArray[i, j], nodeArray[i + 1, j + 1]);
                        edgeList.AddLast(edge);
                        counter++;
                    }
                    if (i + 1 < xMax && j - 1 >= 0)
                    {
                        edge = new CEdge(counter, nodeArray[i, j], nodeArray[i + 1, j - 1]);
                        edgeList.AddLast(edge);
                        counter++;
                    }

                }
            }

            foreach (CNode node in nodeArray)
            {
                node.RemoveNullNodes();
            }
            return edgeList;
        }

        public LinkedList<CEdge> CalculateEdges(LinkedList<CNode> nodeList)
        {

            double shorter = nodeList.ElementAt(1).Point.X - nodeList.ElementAt(0).Point.X;
            shorter = shorter + shorter / 2;
            foreach (CNode node in nodeList)
            {
                node.SetEdgeCount(nodeList.Count() - 1);
            }

            int counter = 0;
            int count = (nodeList.Count() * (nodeList.Count() - 1)) / 2;
            LinkedList<CEdge> edgeList = new LinkedList<CEdge>();
            for (int i = 0; i < nodeList.Count(); i++)
            {
                for (int j = i + 1; j < nodeList.Count(); j++)
                {
                    CEdge edge = new CEdge(counter, nodeList.ElementAt(i), nodeList.ElementAt(j));
                    if (edge.Distance < 90)
                    {

                        edgeList.AddLast(edge);
                    }
                    else
                    {
                        edge.Destroy();
                    }
                    counter++;
                }
            }

            foreach (CNode node in nodeList)
            {
                node.RemoveNullNodes();
            }
            return edgeList;
        }

         public CEdge[] CalculateCompleteEdges(LinkedList<CNode> nodeArray)
        {
            foreach (CNode node in nodeArray)
            {
                node.SetEdgeCount(nodeArray.Count() - 1);
            }

            int counter = 0;
            int count = (nodeArray.Count() * (nodeArray.Count() - 1)) / 2;
            CEdge[] edgeArray = new CEdge[count];
            for (int i = 0; i < nodeArray.Count(); i++)
            {
                for (int j = i + 1; j < nodeArray.Count(); j++)
                {
                    edgeArray[counter] = new CEdge(counter, nodeArray.ElementAt(i), nodeArray.ElementAt(j));
                    counter++;
                }
            }
            return edgeArray;
        }
        public LinkedList<CEdgeLine> CalculateEdgesLines(LinkedList<CNode> nodeList)
        {
            LinkedList<CEdge> list = this.CalculateEdges(nodeList);
            LinkedList<CEdgeLine> lineList = new LinkedList<CEdgeLine>();
            foreach (CEdge edge in list)
            {
                lineList.AddLast(edge.ToCEdgeLine());
            }
            return lineList;
        }

       

        private double CalculateDistance(CNode node1, CNode node2)
        {
            double xDiff = node1.Point.X - node2.Point.X;
            double yDiff = node1.Point.Y - node2.Point.Y;

            double x2 = xDiff * xDiff;
            double y2 = yDiff * yDiff;

            double c2 = x2 + y2;
            double result = Math.Round(Math.Sqrt(c2), 3);
            return result;
        }
    }
}
